
#include "ulib/graphics/draw/draw.hh"
#include <string.h>


template <class T> void DrawPoint(Bitmap<T>& bm,int x,int y,T color)
{
  if (x<0 || y<0) return;
  if (x>=bm.AskWidth() || y>=bm.AskHeight()) return;

  bm.AskFrame()[y][x]=color;
}


template <class T> void DrawLine(Bitmap<T>& bm,int x1,int y1,int x2,int y2,T color)
{
  int vx = x2-x1;
  int vy = y2-y1;

  DrawPoint(bm,x1,y1,color);

  if (abs(vx)>abs(vy))
    {
      for (int x=0;x!=vx;x+=sign(vx))
	DrawPoint(bm , x1+x , vy*x/vx+y1 , color);
    }
  else
    {
      for (int y=0;y!=vy;y+=sign(vy))
	DrawPoint(bm , x1+vx*y/vy , y1+y , color);
    }
}


template <class T> void DrawDottedLine(Bitmap<T>& bm,int x1,int y1,int x2,int y2,T color)
{
  int vx = x2-x1;
  int vy = y2-y1;
  DrawPoint(bm,x1,y1,color);

  if (abs(vx)>abs(vy))
    {
      for (int x=0;x!=vx;x+=4*sign(vx))
	DrawPoint(bm , x1+x , vy*x/vx+y1 , color);
    }
  else
    {
      for (int y=0;y!=vy;y+=4*sign(vy))
	DrawPoint(bm , x1+vx*y/vy , y1+y , color);
    }
}


template <class T> void Clear(Bitmap<T>& bm,T color)
{
  T*const* p = bm.AskFrame();
  
  if (sizeof(T)==1)
    {
      for (int y=0;y<bm.AskHeight();y++)
	memset(p[y],color,bm.AskWidth());
    }
  else
    {
      for (int y=0;y<bm.AskHeight();y++)
	for (int x=0;x<bm.AskWidth();x++)
	  p[y][x] = color;
    }
}


void CopyInnerBorder(const Image<Pixel>& src,Image<Pixel>& dst,int hwidth,int vwidth)
{
  for (int i=0;i<4;i++)
    {
            Bitmap<Pixel>& dstbm = dst.AskBitmap      ((Image<Pixel>::BitmapChannel)i);
      const Bitmap<Pixel>& srcbm = src.AskBitmap_const((Image<Pixel>::BitmapChannel)i);


      // Skip empty bitmaps.

      assert(srcbm.IsEmpty() == dstbm.IsEmpty());

      if (srcbm.IsEmpty())
	continue;


      // Copy border.

      int w = srcbm.AskWidth();
      int h = srcbm.AskHeight();

      assert(dstbm.AskWidth()  == w);
      assert(dstbm.AskHeight() == h);

            Pixel*const* dp = dstbm.AskFrame();
      const Pixel*const* sp = srcbm.AskFrame_const();

      // Horizontal stripes.

      for (int x=0;x<w;x++)
	for (int y=0;x<vwidth;y++)
	  {
	    dp[    y][x]=sp[    y][x];
	    dp[h-1-y][x]=sp[h-1-y][x];
	  }

      // Vertical stripes.
      
      for (int y=vwidth;y<h-vwidth;y++)
	for (int x=0;x<hwidth;x++)
	  {
	    dp[y][    x]=sp[y][    x];
	    dp[y][w-1-x]=sp[y][w-1-x];
	  }
    }
}

void EnhanceImageWithBorder(Image<Pixel>& img,int borderwidth,bool exactsize)
{
  for (int i=0;i<4;i++)
    {
      Bitmap<Pixel>& srcbm = img.AskBitmap((Image<Pixel>::BitmapChannel)i);
      Bitmap<Pixel>* dstbm;
      Bitmap<Pixel>  newbm;

      if (srcbm.IsEmpty())
	continue;

      int w = srcbm.AskWidth(), h = srcbm.AskHeight();

      bool replacebm;

      if ((!exactsize && srcbm.AskBorderWidth() >= borderwidth) ||
	  ( exactsize && srcbm.AskBorderWidth() == borderwidth))
	{
	  dstbm = &srcbm;
	  replacebm=false;
	}
      else
	{
	  newbm.Create(w,h,1,1,borderwidth);
	  dstbm = &newbm;
	  replacebm = true;
	}

      const Pixel*const* sp = srcbm.AskFrame_const();
            Pixel*const* dp = dstbm->AskFrame();

      for (int y=0;y<h;y++)
	for (int x=0;x<w;x++)
	  {
	    dp[y][x] = sp[y][x];
	  }

      for (int y=0;y<h;y++)
	for (int x=0;x<borderwidth;x++)
	  {
	    dp[y][-1-x]=dp[y][0];
	    dp[y][w+x] =dp[y][w-1];
	  }

      for (int x=-borderwidth;x<w+borderwidth;x++)
	for (int y=0;y<borderwidth;y++)
	  {
	    dp[-1-y][x]=dp[0][x];
	    dp[h+y][x] = dp[h-1][x];
	  }

      if (replacebm)
	img.ReplaceBitmap((Image<Pixel>::BitmapChannel)i,*dstbm);
    }
}


template void DrawPoint     (Bitmap<Pixel>&,int x,int y,Pixel color);
template void DrawLine      (Bitmap<Pixel>&,int x1,int y1,int x2,int y2,Pixel color);
template void DrawDottedLine(Bitmap<Pixel>&,int x1,int y1,int x2,int y2,Pixel color);
template void Clear         (Bitmap<Pixel>&,Pixel color);

template void Clear         (Bitmap<bool>&,bool color);
