
#include "ulib/graphics/lowlevel/yuv2rgb16.hh"
#include <iostream.h>
#include <iomanip.h>

void i2r_16bit::SetMask(uint16 redmask,uint16 greenmask,uint16 bluemask)
{
  rmask=redmask;
  gmask=greenmask;
  bmask=bluemask;

  int bits;
  CalcBitsShift(redmask  ,bits,rshift); rshift -= 8-bits;
  CalcBitsShift(greenmask,bits,gshift); gshift -= 8-bits;
  CalcBitsShift(bluemask ,bits,bshift); bshift -= 8-bits;

  rshift -= 8;
  gshift -= 8;
  bshift -= 8;

  rshift = -rshift;
  gshift = -gshift;
  bshift = -bshift;
}


void i2r_16bit::Transform(const Image_YUV<Pixel>& img,uint8* mem)
{
  ImageParam_YUV param;
  img.GetParam(param);

  assert(param.chroma==Chroma420);

  const Pixel*const* yp = img.AskFrameY_const();
  const Pixel*const* up = img.AskFrameU_const();
  const Pixel*const* vp = img.AskFrameV_const();

  int chr_w, chr_h;

  param.GetChromaSizes(chr_w,chr_h);

  bool fastversion = (rshift==0 &&  // red does not need shifting
                      bshift>=0);   // blue will be shifted right -> no need to mask it out

  if (fastversion)
    {
      for (int cy=0;cy<chr_h;cy++)
        {
          const Pixel*  yptr1 = yp[2*cy  ];
          const Pixel*  yptr2 = yp[2*cy+1];
          const Pixel*  uptr  = up[cy];
          const Pixel*  vptr  = vp[cy];
          uint16* membuf16a = ((uint16*)(mem+2*cy*d_bpl));
          uint16* membuf16b = ((uint16*)(mem+2*cy*d_bpl+d_bpl));

          for (int cx=0;cx<chr_w;cx++)
            {
              int u=((int)*uptr++) -128;
              int v=((int)*vptr++) -128;
              
              int r0 = (int)(         + 409*v);
              int g0 = (int)( - 100*u - 208*v);
              int b0 = (int)( + 516*u        );

              int val;
              int yy=(((int)*yptr1++) -16)*298;

              int red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val = rmask; } else { val = (red)&rmask; }
              int green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              int blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift); }
              *membuf16a++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));

              yy=(((int)*yptr1++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val = rmask; } else { val = (red)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift); }
              *membuf16a++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
              
              yy=(((int)*yptr2++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val = rmask; } else { val = (red)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift); }
              *membuf16b++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
              
              yy=(((int)*yptr2++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val = rmask; } else { val = (red)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift); }
              *membuf16b++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
            }
        }
    }
  else
    {
      for (int cy=0;cy<chr_h;cy++)
        {
          const Pixel*  yptr1 = yp[2*cy  ];
          const Pixel*  yptr2 = yp[2*cy+1];
          const Pixel*  uptr  = up[cy];
          const Pixel*  vptr  = vp[cy];
          uint16* membuf16a = ((uint16*)(mem+2*cy*d_bpl));
          uint16* membuf16b = ((uint16*)(mem+2*cy*d_bpl+d_bpl));

          for (int cx=0;cx<chr_w;cx++)
            {
              int u=((int)*uptr++) -128;
              int v=((int)*vptr++) -128;
              
              int r0 = (int)(         + 409*v);
              int g0 = (int)( - 100*u - 208*v);
              int b0 = (int)( + 516*u        );
              
              int val;
              int yy=(((int)*yptr1++) -16)*298;
              int red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val=rmask; } else { val = (red>>rshift)&rmask; }
              int green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              int blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift)&bmask; }
              *membuf16a++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
              
              yy=(((int)*yptr1++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val=rmask; } else { val = (red>>rshift)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift)&bmask; }
              *membuf16a++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
              
              yy=(((int)*yptr2++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val=rmask; } else { val = (red>>rshift)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift)&bmask; }
              *membuf16b++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
              
              yy=(((int)*yptr2++) -16)*298;
              red   = r0+yy;
              if (red<=0) { val=0; } else if (red>0xff00) { val=rmask; } else { val = (red>>rshift)&rmask; }
              green = g0+yy;
              if (green<=0) { } else if (green>0xff00) { val|=gmask; } else { val |= (green>>gshift)&gmask; }
              blue  = b0+yy;
              if (blue<=0) { } else if (blue>=0xff00) { val |= bmask; } else { val |= (blue>>bshift)&bmask; }
              *membuf16b++ = (littleendian ? ToLittleEndian((uint16)val) : ToBigEndian((uint16)val));
            }
        }
    }
}
